//package com.icl.saxon;


import com.icl.saxon.*;

import com.icl.saxon.tree.TreeBuilder;
import com.icl.saxon.tinytree.TinyBuilder;
import com.icl.saxon.om.Builder;
import com.icl.saxon.om.Navigator;
import com.icl.saxon.om.DocumentInfo;
import com.icl.saxon.om.Namespace;
import com.icl.saxon.om.NamePool;
import com.icl.saxon.expr.*;
import com.icl.saxon.style.*;
import com.icl.saxon.output.*;
import com.icl.saxon.trace.*;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.*;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

import java.util.*;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;

import javax.xml.transform.*;
import javax.xml.transform.sax.*;
import javax.xml.transform.stream.*;

  
public class StyleSheet  {

	protected TransformerFactoryImpl factory = new TransformerFactoryImpl();
	
    protected NamePool namePool = NamePool.getDefaultNamePool();
    public static final boolean showTime = true;
    int repeat = 1;
        
    
    public static void main (String args[])
        throws java.lang.Exception
    {
        (new StyleSheet()).doMain(args, new StyleSheet() );
    }


      public void doMain(String args[], StyleSheet app) {
        
        String sourceFileName = null;
        String styleFileName = null;

        File sourceFile = null;
        File styleFile = null;

        boolean useURLs = false;
        Properties outputProperties = new Properties();
        String outputFileName = null;
       
        try {

      sourceFileName = args[0];
      styleFileName = args[1];

      Source sourceInput = null;

      sourceFile = new File(sourceFileName);

      ExtendedInputSource   eis = new ExtendedInputSource(sourceFile);
      sourceInput = new SAXSource(factory.getSourceParser(), eis);
      eis.setEstimatedLength((int)sourceFile.length());


      long startTime = (new Date()).getTime();

      File sheetFile = new File(styleFileName);

      eis = new ExtendedInputSource(sheetFile);
      Source styleSource = new SAXSource(factory.getStyleParser(), eis);

      Templates sheet = factory.newTemplates(styleSource);

        if (showTime) {
            long endTime = (new Date()).getTime();
            System.err.println("Preparation time: " + (endTime-startTime) + " milliseconds");
            startTime = endTime;
        }
                
         processFile(sourceInput, sheet);

        } catch (TransformerException err) {
            quit("Transformation failed: " + err.getMessage());
        } catch (Exception err2) {
            err2.printStackTrace();
        }
        System.exit(0);
    }

    /**
    * Exit with a message
    */
    protected static void quit(String message) {
        System.err.println(message);
        System.exit(2);
    }

 

    /**
    * Process a single file using a supplied stylesheet
    */
    public void processFile (
        Source source, Templates sheet )
        throws TransformerException {

        for (int r=0; r<repeat; r++) {
            long startTime = (new Date()).getTime();
            Transformer instance = sheet.newTransformer();

            Result result = new StreamResult(System.out);
            
            try {
                instance.transform(source, result);
            } catch (TransformerException err) {
                throw new TransformerException("run-time errors were reported");
            }

            if (showTime) {
                long endTime = (new Date()).getTime();
                System.err.println("Execution   time: " + (endTime-startTime) + " milliseconds");
                startTime = endTime;
            }
        }
    }


}