//package com.jclark.xsl.sax;
//import java.io.OutputStream;
//import java.io.FileOutputStream;
//import com.sun.xml.tree.XmlDocument;

import com.jclark.xsl.sax.*;
import java.util.*;

import org.xml.sax.*;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileDescriptor;
import java.net.URL;

public class DriverKorl {
  static class ErrorHandlerImpl implements ErrorHandler {
    public void warning(SAXParseException e) {
      printSAXParseException(e);
    }
    public void error(SAXParseException e) {
      printSAXParseException(e);
    }
    public void fatalError(SAXParseException e) throws SAXException {
      throw e;
    }
  }

  
//=======================================
  public static void main (String args[])
  throws Exception  {
    try {
    XSLProcessorImpl xsl = new XSLProcessorImpl();
    setParser(xsl);
    xsl.setErrorHandler(new ErrorHandlerImpl());

    OutputMethodHandlerImpl outputMethodHandler = new OutputMethodHandlerImpl(xsl);
    xsl.setOutputMethodHandler(outputMethodHandler);
    boolean succeeded = true;

    File in = new File(args[0]);
    File stylesheet = new File(args[1]);
    File out = new File(args[2]);

    succeeded = transformFile(xsl, outputMethodHandler, in, stylesheet, out);

    if (!succeeded)
      System.exit(1);
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      System.exit(0);
    }
  }


  public static boolean transformFile(XSLProcessor xsl,
			       OutputMethodHandlerImpl outputMethodHandler,
			       File inputFile,
			       File stylesheetFile,
			       File outputFile) {
    Destination dest;
    if (outputFile == null)
      dest = new FileDescriptorDestination(FileDescriptor.out);
    else
      dest = new FileDestination(outputFile);
    outputMethodHandler.setDestination(dest);
    return transform(xsl,
		     fileInputSource(stylesheetFile),
		     fileInputSource(inputFile));
  }


  public static boolean transform(XSLProcessor xsl,
			   InputSource stylesheetSource,
			   InputSource inputSource) {
    try {
      long startTime = (new Date()).getTime();
    
      xsl.loadStylesheet(stylesheetSource);
      long endTime = (new Date()).getTime();
                  System.err.println("Preparation time: " + (endTime-startTime) + " milliseconds");
                  startTime = endTime;
      xsl.parse(inputSource);
      endTime = (new Date()).getTime();
                      System.err.println("Execution   time: " + (endTime-startTime) + " milliseconds");
                      startTime = endTime;
      return true;
    }
    catch (SAXParseException e) {
      printSAXParseException(e);
    }
    catch (SAXException e) {
      System.err.println(e.getMessage());
    }
    catch (IOException e) {
      System.err.println(e.toString());
    }
    return false;
  }

  public static void printSAXParseException(SAXParseException e) {
    String systemId = e.getSystemId();
    int lineNumber = e.getLineNumber();
    if (systemId != null)
      System.err.print(systemId + ":");
    if (lineNumber >= 0)
      System.err.print(lineNumber + ":");
    if (systemId != null || lineNumber >= 0)
      System.err.print(" ");
    System.err.println(e.getMessage());
  }

  public static void setParser(XSLProcessorImpl xsl) {
    String parserClass = System.getProperty("com.jclark.xsl.sax.parser");
    if (parserClass == null)
      parserClass = System.getProperty("org.xml.sax.parser");
    if (parserClass == null)
      parserClass = "com.jclark.xml.sax.CommentDriver";
    try {
      Object parserObj = Class.forName(parserClass).newInstance();
      if (parserObj instanceof XMLProcessorEx)
	xsl.setParser((XMLProcessorEx)parserObj);
      else
	xsl.setParser((Parser)parserObj);
      return;
    }
    catch (ClassNotFoundException e) {
      System.err.println(e.toString());
    }
    catch (InstantiationException e) {
      System.err.println(e.toString());
    }
    catch (IllegalAccessException e) {
      System.err.println(e.toString());
    }
    catch (ClassCastException e) {
      System.err.println(parserClass + " is not a SAX driver");
    }
    System.exit(1);
  }

  /**
   * Generates an <code>InputSource</code> from a file name.
   */
  public static InputSource fileInputSource(String str) {
    return fileInputSource(new File(str));
  }

  static public InputSource fileInputSource(File file) {
    String path = file.getAbsolutePath();
//
//    String fSep = System.getProperty("file.separator");
//    if (fSep != null && fSep.length() == 1)
//      path = path.replace(fSep.charAt(0), '/');
//    if (path.length() > 0 && path.charAt(0) != '/')
//      path = '/' + path;
//
    try {
      return new InputSource(new URL("file", "", path).toString());
    }
    catch (java.net.MalformedURLException e) {
      /* According to the spec this could only happen if the file
	 protocol were not recognized. */
      throw new Error("unexpected MalformedURLException");
    }
  }

}